@@ -1,7 +1,7 @@
 /**
  * Handles building of classes
  *
- *  Copyright (C) 2011, 2012, 2013, 2014 Free Software Foundation, Inc.
+ *  Copyright (C) 2011, 2012, 2013, 2014, 2015 Free Software Foundation, Inc.
  *
  *  This file is part of GNU ease.js.
  *
@@ -144,15 +144,6 @@ function ClassBuilder( warn_handler, member_builder, visibility_factory )
      */
     this._instanceId = 0;
 
-    /**
-     * Set to TRUE when class is in the process of being extended to ensure that
-     * a constructor can be instantiated (to use as the prototype) without
-     * invoking the class construction logic
-     *
-     * @type {boolean}
-     */
-    this._extending = false;
-
     /**
      * A flag to let the system know that we are currently attempting to access
      * a static property from within a method. This means that the caller should
@@ -242,6 +233,30 @@ exports.getMeta = function( cls )
 }
 
 
+/**
+ * Allow OBJ to assume an identity as a class
+ *
+ * This is useful to use objects in situations where classes are expected,
+ * as it eliminates the need for handling of special cases.
+ *
+ * This is intended for internal use---there are no guarantees as to what
+ * methods ease.js may expect that a class-like object incorporate.  That
+ * guarantee may exist in the future, but until then, stay away.
+ *
+ * @param {Object} obj object to masquerade as an ease.js class
+ *
+ * @return {Object} OBJ
+ */
+exports.masquerade = function( obj )
+{
+    // XXX: this is duplicated; abstract
+    util.defineSecureProp( obj, _priv, {} );
+
+    createMeta( obj, exports.ClassBase );
+    return obj;
+};
+
+
 /**
  * Determines if the class is an instance of the given type
  *
@@ -321,9 +336,6 @@ exports.prototype.build = function extend( _, __ )
 {
     var build = this;
 
-    // ensure we'll be permitted to instantiate abstract classes for the base
-    this._extending = true;
-
     var a         = arguments,
         an        = a.length,
         props     = ( ( an > 0 ) ? a[ an - 1 ] : 0 ) || {},
@@ -339,18 +351,19 @@ exports.prototype.build = function extend( _, __ )
             props:   this._memberBuilder.initMembers(),
         },
 
-        meta = exports.getMeta( base ) || {},
+        // constructor may be different than base
+        pmeta = exports.getMeta( prototype.constructor ) || {},
 
         abstract_methods =
-            util.clone( meta.abstractMethods )
+            util.clone( pmeta.abstractMethods )
             || { __length: 0 },
 
         virtual_members =
-            util.clone( meta.virtualMembers )
+            util.clone( pmeta.virtualMembers )
             || {}
     ;
 
-    // prevent extending final classes
+    // prevent extending final classes (TODO: abstract this check)
     if ( base.___$$final$$ === true )
     {
         throw Error(
@@ -464,7 +477,7 @@ exports.prototype.build = function extend( _, __ )
     util.defineSecureProp( prototype, '__self', new_class.___$$svis$$ );
 
     // create internal metadata for the new class
-    var meta = createMeta( new_class, base );
+    var meta = createMeta( new_class, base, pmeta );
     meta.abstractMethods = abstract_methods;
     meta.virtualMembers  = virtual_members;
     meta.name            = cname;
@@ -476,15 +489,13 @@ exports.prototype.build = function extend( _, __ )
     // (intended for use in prototype chains)
     new_class.asPrototype = function()
     {
-        build._extending = true;
-        var inst = new_class();
-        build._extending = false;
+        new_class[ _priv ].extending = true;
+        var inst = new new_class();
+        new_class[ _priv ].extending = false;
+
         return inst;
     };
 
-    // we're done with the extension process
-    this._extending = false;
-
     return new_class;
 };
 
@@ -498,7 +509,9 @@ exports.prototype._getBase = function( base )
         // constructor (we could also check to ensure that the return value of
         // the constructor is an object, but that is not our concern)
         case 'function':
-            return new base();
+            return ( base[ _priv ] )
+                ? base.asPrototype()
+                : new base();
 
         // we can use objects as the prototype directly
         case 'object':
@@ -841,8 +854,8 @@ exports.prototype.createCtor = function( cname, abstract_methods, members )
  */
 exports.prototype.createConcreteCtor = function( cname, members )
 {
-    var args  = null,
-        _self = this;
+    var args    = null,
+        _self   = this;
 
     /**
      * Constructor function to be returned
@@ -872,7 +885,7 @@ exports.prototype.createConcreteCtor = function( cname, members )
         // If we're extending, we don't actually want to invoke any class
         // construction logic. The above is sufficient to use this class in a
         // prototype, so stop here.
-        if ( _self._extending )
+        if ( ClassInstance[ _priv ].extending )
         {
             return;
         }
@@ -968,7 +981,7 @@ exports.prototype.createAbstractCtor = function( cname )
 
     var __abstract_self = function()
     {
-        if ( !_self._extending )
+        if ( !__abstract_self[ _priv ].extending )
         {
             throw Error(
                 "Abstract class " + ( cname || '(anonymous)' ) +
@@ -1255,18 +1268,23 @@ exports.prototype.attachStatic = function( ctor, members, base, inheriting )
 /**
  * Initializes class metadata for the given class
  *
+ * DYNMETA is used only when CPARENT's metadata are flagged as "lazy",
+ * meaning that the data are not available at the time of its definition,
+ * but are available now as DYNMETA.
+ *
  * @param  {Function}  func     class to initialize metadata for
  * @param  {Function}  cparent  class parent
+ * @param  {?Object}   dynmeta  dynamic metadata
  *
  * @return  {undefined}
  *
  * Suppressed due to warnings for use of __cid
  * @suppress {checkTypes}
  */
-function createMeta( func, cparent )
+function createMeta( func, cparent, dynmeta )
 {
     var id          = func.__cid,
-        parent_meta = ( ( cparent.__cid )
+        parent_meta = ( cparent[ _priv ]
             ? exports.getMeta( cparent )
             : undefined
         );
@@ -1274,7 +1292,13 @@ function createMeta( func, cparent )
     // copy the parent prototype's metadata if it exists (inherit metadata)
     if ( parent_meta )
     {
-        return func[ _priv ].meta = util.clone( parent_meta, true );
+        return func[ _priv ].meta = util.clone(
+            // "lazy" metadata are unavailable at the time of definition
+            parent_meta._lazy
+                ? dynmeta
+                : parent_meta,
+            true
+        );
     }
 
     // create empty
@@ -1461,4 +1485,3 @@ function attachFlags( ctor, props )
     // (v8 performance)
     props.___$$final$$ = props.___$$abstract$$ = undefined;
 }
-
