@@ -1,7 +1,7 @@
 /**
  * Provides system for code reuse via traits
  *
- *  Copyright (C) 2014 Free Software Foundation, Inc.
+ *  Copyright (C) 2014, 2015 Free Software Foundation, Inc.
  *
  *  This file is part of GNU ease.js.
  *
@@ -97,8 +97,50 @@ function _createStaging( name )
 }
 
 
-Trait.extend = function( dfn )
+Trait.extend = function( /* ... */ )
 {
+    var an           = arguments.length,
+        dfn          = arguments[ an - 1 ],
+        has_ext_base = ( an > 1 ),
+        ext_base     = ( has_ext_base ) ? arguments[ 0 ] : null;
+
+    if ( an > 2 )
+    {
+        throw Error(
+            "Unexpected number of arguments to Trait.extend"
+        );
+    }
+
+    if ( has_ext_base )
+    {
+        var basetype = typeof ext_base;
+
+        if ( ( ext_base === null )
+             || !( ( basetype === 'object' )
+                   || ( basetype === 'function' )
+        ) )
+        {
+            throw TypeError(
+                "Trait cannot extend base of type '" + basetype + "'"
+            );
+        }
+
+        // prevent extending final classes (TODO: abstract this check; see
+        // also ClassBuilder)
+        if ( ext_base.___$$final$$ === true )
+        {
+            throw TypeError(
+                "Trait cannot extend final class"
+            );
+        }
+
+        // TODO: this is intended to be temporary; see Trait/ClassExtendTest
+        if ( module.exports.isTrait( ext_base ) )
+        {
+            throw TypeError( "Traits cannot extend other traits" );
+        }
+    }
+
     // we may have been passed some additional metadata
     var meta = ( this || {} ).__$$meta || {};
 
@@ -168,10 +210,11 @@ Trait.extend = function( dfn )
     // and here we can see that traits are quite literally abstract classes
     var tclass = base.extend( dfn );
 
-    Trait.__trait  = type;
-    Trait.__acls   = tclass;
-    Trait.__ccls   = null;
-    Trait.toString = function()
+    Trait.__trait   = type;
+    Trait.__acls    = tclass;
+    Trait.__ccls    = null;
+    Trait.__extbase = ext_base;
+    Trait.toString  = function()
     {
         return ''+name;
     };
@@ -198,6 +241,41 @@ Trait.extend = function( dfn )
 };
 
 
+/**
+ * Validate whether mixin is permitted
+ *
+ * If a mixee (the trait being mixed in) extends some type S, then a
+ * contract has been created mandating that that trait may only be mixed
+ * into something of type S; a `TypeError` will be thrown if this contract
+ * is violated.
+ *
+ * @param  {Class}  base  mixor (target of mixin)
+ * @param  {Trait}  T     mixee (trait being mixed in)
+ *
+ * @return  {undefined}
+ *
+ * @throws  {TypeError}  on type contract violation
+ */
+function _validateMixin( base, T )
+{
+    if ( !T.__extbase )
+    {
+        return;
+    }
+
+    // TODO: isSubtypeOf
+    if ( !( ( T.__extbase === base )
+            || ClassBuilder.isInstanceOf( T.__extbase, base.asPrototype() )
+    ) )
+    {
+        throw TypeError(
+            "Cannot mix trait " + T.toString() + " into " + base.toString() +
+                "; mixor must be of type " + T.__extbase.toString()
+        );
+    }
+}
+
+
 /**
  * Retrieve a string representation of the trait type
  *
@@ -530,12 +608,14 @@ function createVirtProxy( acls, dfn )
  * @param  {Trait}   trait  trait to mix in
  * @param  {Object}  dfn    definition object to merge into
  * @param  {Array}   tc     trait class context
- * @param  {Class}   base   target supertyep
+ * @param  {Class}   base   target supertype
  *
  * @return  {Object}  dfn
  */
 function mixin( trait, dfn, tc, base )
 {
+    _validateMixin( base, trait );
+
     // the abstract class hidden within the trait
     var acls = trait.__acls;
 
